/////////////////////////////////////////////////////////////////////////////
// Name:        About.h
// Purpose:     About dialog
// Author:      Alex Thuering
// Created:     6.11.2003
// RCS-ID:      $Id: About.h,v 1.17 2008/03/02 19:03:17 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include <wx/wx.h>
#include "Version.h" //APP_VERSION

const wxString APP_MAINT		= _T("Alex Thuering");
const wxString APP_NAME			= _T("DVDStyler");
const wxString APP_LICENCE		= _T("GPL");
const wxString APP_COPYRIGTH	= _T("(c) 2008 Alex Thuering");
const wxString APP_WEBSITE		= _T("http://www.dvdstyler.de");

class About: public wxDialog
{
  public:
    About (wxWindow *parent);
};
