/////////////////////////////////////////////////////////////////////////////
// Name:        DVDPropDlg.h
// Purpose:     DVD properties dialog
// Author:      Alex Thuering
// Created:     7.03.2005
// RCS-ID:      $Id: DVDPropDlg.h,v 1.2 2005/10/18 08:12:04 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#ifndef DVD_PROP_DLG_H
#define DVD_PROP_DLG_H

#include <wx/wx.h>
#include <wxVillaLib/PropDlg.h>
#include "DVD.h"

class DVDPropDlg: public wxPropDlg
{
  public:
    DVDPropDlg(wxWindow *parent, DVD* dvd);
	virtual ~DVDPropDlg() {}

  protected:
	DVD* m_dvd;
	void CreatePropPanel(wxSizer* sizer);
	bool SetValues();
};

#endif // DVD_PROP_DLG_H
