/////////////////////////////////////////////////////////////////////////////
// Name:        NewPropjectDlg.h
// Purpose:     New project dialog
// Author:      Alex Thuering
// Created:	29.10.2006
// RCS-ID:      $Id: NewProjectDlg.h,v 1.3 2006/12/06 14:53:37 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef NEW_PROJECT_PROP_DLG_H
#define NEW_PROJECT_PROP_DLG_H

#include <wx/wx.h>
#include <wxVillaLib/PropDlg.h>
#include "Menu.h"

class NewProjectDlg: public wxPropDlg
{
  public:
	NewProjectDlg(wxWindow *parent);
	virtual ~NewProjectDlg() {}
	
	wxString GetVolumeName();
	VideoFormat GetVideoFormat();
	AudioFormat GetAudioFormat();
	
  protected:
	void CreatePropPanel(wxSizer* sizer);
	bool SetValues() { return true; }
};

#endif // NEW_PROJECT_PROP_DLG_H
