Document: dvdstyler
Title: Debian dvdstyler Manual
Author: <insert document author here>
Abstract: This manual describes what dvdstyler is
 and how it can be used to
 manage online manuals on Debian systems.
Section: unknown

Format: debiandoc-sgml
Files: /usr/share/doc/dvdstyler/dvdstyler.sgml.gz

Format: postscript
Files: /usr/share/doc/dvdstyler/dvdstyler.ps.gz

Format: text
Files: /usr/share/doc/dvdstyler/dvdstyler.text.gz

Format: HTML
Index: /usr/share/doc/dvdstyler/html/index.html
Files: /usr/share/doc/dvdstyler/html/*.html
