/////////////////////////////////////////////////////////////////////////////
// Name:        Languages.h
// Purpose:     Languages dialog
// Author:      Alex Thuering
// Created:	23.05.2003
// RCS-ID:      $Id: Languages.h,v 1.4 2006/12/06 14:53:37 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef DVDSTYLER_LANGUAGES_H
#define DVDSTYLER_LANGUAGES_H

#include <wx/wx.h>

int ChooseLanguage();
wxArrayString GetLangNames();
wxString GetLangName(int code);
int GetLangCode(int index);

#endif // DVDSTYLER_LANGUAGES_H
