/////////////////////////////////////////////////////////////////////////////
// Name:        Config.h
// Purpose:     Configuration
// Author:      Alex Thuering
// Created:	27.03.2003
// RCS-ID:      $Id: Config.cpp,v 1.4 2006/12/06 14:53:37 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "Config.h"

Config s_config;

wxRect Config::GetMainWinLocation()
{
  wxRect rect;
  rect.x = cfg->Read(_T("MainWin/x"), -1);
  rect.y = cfg->Read(_T("MainWin/y"), -1);
  rect.width = cfg->Read(_T("MainWin/width"), -1);
  rect.height = cfg->Read(_T("MainWin/height"), -1);
  return rect;
}

void Config::SetMainWinLocation(wxRect rect)
{
  cfg->Write(_T("MainWin/x"), rect.x);
  cfg->Write(_T("MainWin/y"), rect.y);
  cfg->Write(_T("MainWin/width"), rect.width);
  cfg->Write(_T("MainWin/height"), rect.height);
}
