/////////////////////////////////////////////////////////////////////////////
// Name:        SettingsDlg.h
// Purpose:     Setting Dlg
// Author:      Alex Thuering
// Created:     27.03.2004
// RCS-ID:      $Id: SettingsDlg.h,v 1.4 2008/03/02 19:01:48 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#ifndef SETTINGS_DLG_H
#define SETTINGS_DLG_H

#include <wx/wx.h>
#include <wxVillaLib/PropDlg.h>

class SettingsDlg: public wxPropDlg
{
  public:
    SettingsDlg(wxWindow *parent);
	virtual ~SettingsDlg() {}

  protected:
	void CreatePropPanel(wxSizer* sizer);
	bool SetValues();
  
  private:
	wxTextCtrl* m_directoryEdit;
	wxButton* m_directorySelectButton;
	void OnChangeFileBrowserDir(wxCommandEvent& evt);
	DECLARE_EVENT_TABLE()
};

#endif // TEXT_PROP_DLG_H
