/////////////////////////////////////////////////////////////////////////////
// Name:        KeyDlg.h
// Purpose:     Key dialog
// Author:      Alex Thuering
// Created:     18.05.2007
// RCS-ID:      $Id: KeyDlg.h,v 1.2 2007/12/15 19:58:25 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include <wx/wx.h>
#include <wxVillaLib/PropDlg.h>

class KeyDlg: public wxPropDlg
{
  public:
	KeyDlg(wxWindow *parent);
	int ShowModal();
	
  protected:
    void CreateButtonPane(wxSizer* sizer, bool resetButton);
	void CreatePropPanel(wxSizer* sizer);
	bool SetValues();
	void Reset();
};
